#include "Irradiance_first.h"
#include <algorithm>
#include "../src/samplers/hemisphericalsampler.h"
#include "../src/core/Interpolaciones.h"

#include <math.h>


Irradiance_First::Irradiance_First() {
};

Irradiance_First::Irradiance_First(Vector3D bgColor, size_t maxDepth, bool show_point, size_t nSamplesArea_, bool indirectOnly_, size_t nSamplesHemis_):Shader(bgColor,1) {
	path = PathShader(bgColor,1, nSamplesArea_,maxDepth, indirectOnly_);
	jump = 20;
	show_points = false;
	irradianceCache2 = octree(3, Vector3D(0, 0, 3.0), Vector3D(-2.0, 2.0, 0), Vector3D(2.0, -2.0, 6), 2.0, 2.0, 6);
	indirectOnly = indirectOnly_;
	nSamplesHemis = nSamplesHemis_;
};

void Irradiance_First::preprocess(Scene& scene) {

	fillCache(scene.getCamera().film.getWidth(), scene.getCamera().film.getHeight(),scene);

}

void Irradiance_First::fillCache(size_t width_, size_t height_, Scene& scene) {//rellenar cash

    int x, y;
	int i = 0;
	std::vector<Vector3D> displacementSamples = displacementCalc(width_, height_);

    for (size_t x_actual = 0; x_actual < width_; x_actual += jump) {//width_
        for (size_t y_actual = 0; y_actual < height_; y_actual += jump) {

			if(!(width_ < x_actual or height_ < y_actual) ){
				computeRecord(x_actual, y_actual, width_, height_, scene, displacementSamples,i);
				i++;
			}

        }

    }
	
}

std::vector<Vector3D> Irradiance_First::displacementCalc(size_t width_, size_t height_) {

	size_t nSamples1D = (size_t)std::sqrt(nSamplesPP);
	nSamplesPP = nSamples1D * nSamples1D;
	std::vector<Vector3D> displacementSamples;

	double subPixelSize = 1.0 / nSamples1D;

	double dPixel = subPixelSize / 2.0;

	for (size_t sampleIndex = 0; sampleIndex < nSamplesPP; sampleIndex++)
	{
		double xCoord, yCoord;
		xCoord = dPixel + (sampleIndex % nSamples1D) * subPixelSize;
		yCoord = dPixel + (sampleIndex / nSamples1D) * subPixelSize;
		Vector3D displacement(xCoord / width_, yCoord / height_, 0);
		displacementSamples.push_back(displacement);
	}

	return displacementSamples;

}

void Irradiance_First::computeRecord(size_t x, size_t y, size_t width_, size_t height_, Scene& scene, std::vector<Vector3D> displacementSamples, int i) {
	Vector3D pixelColor(0.0);

	Intersection its;
	double harmonicMean = 0.0;
	Vector3D sampleNDC((double)(x) / width_, (double)(y) / height_, 0);
	Vector3D result;
	Ray r = scene.getCamera().generateRay(sampleNDC.x, sampleNDC.y);
	scene.getClosestIntersection(r, its);
	for (size_t sampleIndex = 0; sampleIndex < 1; sampleIndex++)
	{
		Vector3D sampleNDC((double)(x + 0.5) / width_, (double)(y + 0.5) / height_, 0);
		Vector3D irradiance;
		//sampleNDC += Vector3D(0.5,0.5,0)
		
		double u = sampleNDC.x;
		double v = sampleNDC.y;
		Ray cameraRay = scene.getCamera().generateRay(u, v);
		// Compute ray color

		if (scene.getClosestIntersection(r, its))
		{
			/* *************************************************** */
			/* *** Treat the case of diffuse/glossy reflection *** */
			/* *************************************************** */
			if (its.shape->getMaterial().hasDiffuseOrGlossy())
			{
				/* SAMPLE THE HEMISHPERE */
				HemisphericalSampler sampler; // This class gives random points on a hemisphere
				for (size_t n = 0; n < nSamplesHemis; ++n) {
					// Generate a random direction on the hemisphere around the normal at the intersection point
					Vector3D rnd_direction = sampler.getSample(its.normal);
					// Create a new ray with this random direction
					Ray indirectRay(its.itsPoint, rnd_direction, r.depth + 1);
					// Compute the incident radiance from that direction
					Vector3D indirectIncidentRadiance = path.computeColor(indirectRay, scene);

					// ATTENTION: ONLY THE INCIDENT LIGHT SHOULD BE STORED. THAT IS WHY THE ABOVE CODE BECOMES:
					
					irradiance += indirectIncidentRadiance * dot(rnd_direction, its.normal);
					harmonicMean += computeHarmonic(its,indirectRay,scene,x,y);
				}
			}
			// Once all light sources have been taken into account, return the final result
			result = irradiance * (2 * M_PI) / nSamplesHemis;
			harmonicMean = harmonicMean / nSamplesHemis;

			if (its.shape->isEmissiveShape()) {

				if (r.depth == 0) {
					if (!indirectOnly) {
						result += its.shape->getEmission();
					}

				}
			}

		}
		else
		{
			// Found no intersection along the ray
			// Return the background color
			result = bgColor;
		}
	}
	
	//std::cout << harmonicMean << std::endl;
	// Store the pixel color


	IrradianceCacheRecord record;
	record.its = its;
	record.color = result;
	record.harmonicMean = harmonicMean;
	record.i = i;
	record.point2D = Vector3D(x, y, 0);
	//irradianceCache2.pushRecord(record);
	irradianceCache.push_back(record);
}

void Irradiance_First::render(Scene& scene) {
	unsigned int sizeBar = 40;

	// Create a vector with the displacement of each pixel sample
	// Compute the number of samples in one dimension
	size_t nSamples1D = (size_t)std::sqrt(nSamplesPP);
	nSamplesPP = nSamples1D * nSamples1D;
	std::cout << "Using " << nSamplesPP << " samples per pixel" << std::endl;
	// Compute the sub-pixel size
	double subPixelSize = 1.0 / nSamples1D;
	// Compute the sample offset in 1D w.r.t. the subpixel beginning
	double dPixel = subPixelSize / 2.0;

	// Get the film resolution
	size_t resX = scene.getCamera().film.getWidth();
	size_t resY = scene.getCamera().film.getHeight();

	std::vector<Vector3D> displacementSamples;
	for (size_t sampleIndex = 0; sampleIndex < nSamplesPP; sampleIndex++)
	{
		double xCoord, yCoord;
		xCoord = dPixel + (sampleIndex % nSamples1D) * subPixelSize;
		yCoord = dPixel + (sampleIndex / nSamples1D) * subPixelSize;
		Vector3D displacement(xCoord / resX, yCoord / resY, 0);
		displacementSamples.push_back(displacement);
	}

	// Main raytracing loop
	// Out-most loop invariant: we have rendered lin lines 
	for (size_t lin = 0; lin < resY; lin++){
		if (lin % (resY / sizeBar) == 0)
			std::cout << ".";

		// Inner loop invariant: we have rendered col columns 
		for (size_t col = 0; col < resX; col++)
		{
			// Generate image plane sample: (0,0) is the upper left corner
			// Formalize the different spaces used:
			// - Sample space
			// - NDC (normalized device coordinates)
			// - Screen space
			// - Camera space
			Vector3D pixelColor(0.0);

			Vector3D sampleNDC((double)(col) / resX, (double)(lin) / resY, 0);

			Ray cameraRay = scene.getCamera().generateRay(sampleNDC.x, sampleNDC.y);

			/*std::vector<IrradianceCacheRecord> neighbours = near4Neighbour2D(col,lin);
			Interpolaciones es;
			pixelColor += es.computeColor2D1st(col,lin,jump,neighbours);*/

			pixelColor += computeColor3D(cameraRay,scene,col,lin);
			// Store the pixel color
			scene.getCamera().film.setPixelValue(col, lin, pixelColor);

		}
	}
}

std::vector<IrradianceCacheRecord> Irradiance_First::near4Neighbour2D(size_t x, size_t y) {//ferla de n veins en vez de 4 veins
	size_t m_x = jump;
	size_t m_y = jump;

	size_t x_ = x - (x % m_x);
	size_t y_ = y - (y % m_y);

	std::vector<IrradianceCacheRecord> neighbours;
	if (x_ + jump > 512) {
		m_x = 0;
	}if (y_ + jump > 512) {
		m_y = 0;
	}
	Vector3D n0(x_, y_, 0);
	Vector3D n1(x_ + m_x, y_, 0);
	Vector3D n2(x_, y_ + m_y, 0);
	Vector3D n3(x_ + m_x, y_ + m_y, 0);

	for (int i = 0; i < irradianceCache.size(); i++) {
		if (irradianceCache[i].point2D.operator==(n0) or irradianceCache[i].point2D.operator==(n1) or irradianceCache[i].point2D.operator==(n2) or irradianceCache[i].point2D.operator==(n3)) {
			neighbours.push_back(irradianceCache[i]);
		}
	}

	return neighbours;
}

std::vector <IrradianceCacheRecord> Irradiance_First::near_n_Neighbour(Intersection its, int n) {

	std::vector < std::pair<double,int>>  neighboursNpair;
	std::vector <IrradianceCacheRecord> neighboursN;
	double dist;

	for (int i = 0; i < irradianceCache.size(); i++) {
		dist = its.itsPoint.norm(irradianceCache[i].its.itsPoint);
		neighboursNpair.push_back({dist ,i });
	}
	/*neighboursN = irradianceCache2.getRecordsOnOct(its.itsPoint);
	if (neighboursN.size() != 0) {
		std::cout << neighboursN.size() << std::endl;
	}*/
	
	std::sort(neighboursNpair.begin(), neighboursNpair.end());
	for (int i = 0; i < n; i++) {
		neighboursN.push_back(irradianceCache[neighboursNpair[i].second]);
	}
	return neighboursN;

}

Vector3D Irradiance_First::computeColor3D(const Ray& r, Scene& scene, size_t x, size_t y){
	int n = 6;
	Intersection its;
	scene.getClosestIntersection(r, its);

	std::vector<IrradianceCacheRecord> neighbours = near_n_Neighbour(its, n);
	//std::vector<IrradianceCacheRecord> neighbours = near4Neighbour2D(x, y);

	Vector3D E(0, 0, 0);
	Vector3D Esubi(0, 0, 0);
	double wsubi = 0.0;
	double sumBot = 0.0;
	Vector3D sumTop(0, 0, 0);

	Vector3D p = its.itsPoint;
	//Vector3D n = its.normal;

	IrradianceCacheRecord record;
	double dist;
	for (int i = 0; i < neighbours.size(); i++) {
		record = neighbours[i];
		//dist = p.norm(record.its.itsPoint);
		dist = (p - record.its.itsPoint).length();
		if (!show_points and dist == 0) {
			double u, v;
			its.shape->getTextureCoordinates(its.itsPoint, u, v);
			return Utils::multiplyPerCanal(neighbours[i].color, its.shape->getMaterial().getReflectance(its.normal, its.normal, its.normal, u, v));
		}

		//wsubi = 1.0 / ( dist + sqrt(2));
		wsubi = 1.0 / (dist + sqrt(1.0 - its.normal.scalar_product(record.its.normal)));
		//wsubi = 1.0 / (dist/record.harmonicMean + sqrt(1.0 - its.normal.scalar_product(record.its.normal)));
		sumTop = sumTop + (record.color*wsubi);
		sumBot = sumBot + wsubi;
	}
	E = sumTop/sumBot;
	/*for (int i = 0; i < neighbours.size(); i++) {
		sumTop = sumTop + neighbours[i].color;
	}
	E = sumTop / neighbours.size();*/
	double u, v;
	its.shape->getTextureCoordinates(its.itsPoint, u, v);
	return Utils::multiplyPerCanal(E, its.shape->getMaterial().getReflectance(its.normal, its.normal, its.normal, u, v));
}

Vector3D Irradiance_First::computeColor(const Ray& r, Scene& scene) const
{
	return Vector3D();
}

double Irradiance_First::computeHarmonic(Intersection its, Ray r, Scene& scene, int x, int y) {
	Intersection its2;
	double harmonicMean = 0.0;
	if(scene.hasIntersection(r)){
		scene.getClosestIntersection(r, its2);
		harmonicMean = its.itsPoint.norm(its2.itsPoint);
	}
	return harmonicMean;
}
