#include "ortographic.h"

OrtographicCamera::OrtographicCamera(const Matrix4x4 &cameraToWorld_, Film &film_ )
    : Camera(cameraToWorld_, film_)
{

}

Vector3D OrtographicCamera::ndcToCameraSpace(const double u, const double v) const
{
    // Compute the x-coordinate in camera space (taking into account the aspect ratio)
    double x = ((u * 2) - 1) * aspect;
    // Compute the x-coordinate in camera space
    double y = ((v * 2) - 1);

    // Return the position in camera space
    return Vector3D(x, y, 0);
}

// Input in Image plane
Ray OrtographicCamera::generateRay(const double u, const double v) const
{
    // Convert the sample to camera coordinates
    //Vector3D rOrig = ndcToCamera.transformPoint(sample);
    Vector3D rOrig = ndcToCameraSpace(u, v);

    Vector3D rDir(0, 0, 1);

    // Construct the ray
    size_t rayDepth = 0;
    Ray r(rOrig, rDir, rayDepth);

    // Convert the ray to world coordinates
    r = cameraToWorld.transformRay(r);

    // Make sure the ray is normalized
    r.d = r.d.normalized();

    return r;
}
