#include "phong.h"
#define _USE_MATH_DEFINES
#include <math.h>

Phong::Phong() :
    shininess(50)
{
    kdTexture = new Texture(Vector3D(1));
    ksTexture = new Texture(Vector3D(1));
}

Phong::Phong(Vector3D kd_, Vector3D ks_, double shininess_) :
    shininess(shininess_)
{
    kdTexture = new Texture(Vector3D((kd_)));
    ksTexture = new Texture(Vector3D(ks_));
}

Phong::Phong(std::string fileName_, double shininess_) :
    shininess(shininess_)
{
    kdTexture = new Texture(fileName_);
    ksTexture =  kdTexture;
}

bool Phong::hasSpecular() const
{
    // This material does not have specular component
    return false;
}

bool Phong::hasTransmission() const
{
    return false;
}

bool Phong::hasDiffuseOrGlossy() const
{
    return true;
}

Vector3D Phong::getReflectance(const Vector3D &n, const Vector3D &wo, const Vector3D &wi,
                               const double u, const double v) const
{
    if(dot(wi, n) < 0)
        return Vector3D(0,0,0);

    // Compute the perfect reflection direction
    Vector3D wr = n * 2 * dot(n, wo) - wo;

    // Compute the cos of the angle between the incident light direction
    // and the perfect reflection direction
    double cosWrWi = dot(wr, wi);

    Vector3D texValKd = kdTexture->getTextureValue(u, v);
    Vector3D texValKs = ksTexture->getTextureValue(u, v);

    return texValKd / M_PI + texValKs * (shininess+2) / (2*M_PI) * std::pow(cosWrWi, shininess);
}

Phong::~Phong()
{

}
