#include "../shaders/depthshader.h"
#include "../core/utils.h"

DepthShader::DepthShader() :
    color(Vector3D(0.2, 0.5, 0.2)), maxDist(10)
{ }

DepthShader::DepthShader(Vector3D color_, double maxDist_, Vector3D bgColor_, bool useColorMap_) :
    Shader(bgColor_), color(color_), maxDist(maxDist_), useColorMap(useColorMap_)
{ }

Vector3D DepthShader::computeColor(const Ray &r, Scene &scene) const
{
    Intersection its;

	if (scene.getClosestIntersection(r, its))
    {
        double depth = (its.itsPoint - r.o).length();
        if(useColorMap)
        {
            return Utils::scalarToRGB(depth/maxDist);
        } else
        {
			if (depth < maxDist)
				return color * (1.0 - depth / maxDist);
			else
				return bgColor;
        }
    } else
    {
        return bgColor;
    }
}
