#ifndef IRRADIANCE_H
#define IRRADIANCE_H

#include "../shaders/shader.h"
#include "../shaders/pathshader.h"
#include "../core/IrradianceCacheRecord.h"
#include "../core/octree.h"

class irradiance : public Shader{
	private:
		std::vector<IrradianceCacheRecord> irradianceCache;

		octree irradianceCache2;

		PathShader path;

		int jump;
		int nSamplesHemis;
		bool show_points;
		bool indirectOnly;
		bool preprocessDone;
		bool useOctree;

	public:
		irradiance();
		irradiance(Vector3D bgColor, size_t maxDepth, bool show_point, size_t nSamplesArea_, bool indirectOnly_, size_t nSamplesHemis_);

		virtual void preprocess(Scene& scene);

		Vector3D computeColor3D(const Ray& r, Scene& scene, size_t x, size_t y, std::vector<Vector3D> displacementSamples);
		std::vector<IrradianceCacheRecord> near_n_Neighbour(Intersection its, int n);
		std::vector <IrradianceCacheRecord> near_n_Neighbour_Octree(Intersection its, int n);
		virtual void render(Scene& scene);

		double computeHarmonic(Intersection its, Ray indirectRay, Scene& scene, int x, int y);

		Vector3D computeRecord(size_t x, size_t y, size_t width_, size_t height_, Scene& scene, std::vector<Vector3D> displacementSamples);

		virtual Vector3D computeColor(const Ray& r, Scene& scene)const;

		bool sampleisValid(double wsubi, IrradianceCacheRecord i, Intersection its);
};
#endif // IRRADIANCE_H