#include "shader.h"
#include "../../build/Irradiance_first.h"

Shader::Shader() : bgColor(Vector3D(0.0)), nSamplesPP(1)
{ }

Shader::Shader(Vector3D bgColor_) : bgColor(bgColor_), nSamplesPP(1)
{ }

Shader::Shader(Vector3D bgColor_, size_t nSamplesPP_) : bgColor(bgColor_), nSamplesPP(nSamplesPP_)
{ }

void Shader::preprocess(Scene &scene)
{ }

void Shader::postprocess(Scene &scene)
{ }

void Shader::render(Scene &scene) {
	unsigned int sizeBar = 40;
	
	// Create a vector with the displacement of each pixel sample
	// Compute the number of samples in one dimension
	size_t nSamples1D = (size_t)std::sqrt(nSamplesPP);
	nSamplesPP = nSamples1D*nSamples1D;
	std::cout << "Using " << nSamplesPP << " samples per pixel" << std::endl;
	// Compute the sub-pixel size
	double subPixelSize = 1.0 / nSamples1D;
	// Compute the sample offset in 1D w.r.t. the subpixel beginning
	double dPixel = subPixelSize / 2.0;

	// Get the film resolution
	size_t resX = scene.getCamera().film.getWidth();
	size_t resY = scene.getCamera().film.getHeight();

	std::vector<Vector3D> displacementSamples;
	for (size_t sampleIndex = 0; sampleIndex < nSamplesPP; sampleIndex++)
	{
		double xCoord, yCoord;
		xCoord = dPixel + (sampleIndex % nSamples1D) * subPixelSize;
		yCoord = dPixel + (sampleIndex / nSamples1D) * subPixelSize;
		Vector3D displacement(xCoord / resX, yCoord / resY, 0);
		displacementSamples.push_back(displacement);
	}

	// Main raytracing loop
	// Out-most loop invariant: we have rendered lin lines
	for (size_t lin = 0; lin<resY; lin++)
	{
		if (lin % (resY / sizeBar) == 0)
			std::cout << ".";

		// Inner loop invariant: we have rendered col columns
		for (size_t col = 0; col<resX; col++)
		{
			// Generate image plane sample: (0,0) is the upper left corner
			// Formalize the different spaces used:
			// - Sample space
			// - NDC (normalized device coordinates)
			// - Screen space
			// - Camera space
			Vector3D pixelColor(0.0);
			for (size_t sampleIndex = 0; sampleIndex < nSamplesPP; sampleIndex++)
			{
				Vector3D sampleNDC( (double)(col) / resX,
									(double)(lin) / resY,
									0 );

				sampleNDC += displacementSamples.at(sampleIndex);

				double u = sampleNDC.x;
				double v = sampleNDC.y;

				Ray cameraRay = scene.getCamera().generateRay(u, v);

				// Compute ray color
				pixelColor += computeColor(cameraRay, scene);
			}
			// Store the pixel color
			scene.getCamera().film.setPixelValue(col, lin, pixelColor / (double)nSamplesPP);
		}
	}
}