#ifndef SHAPE_H
#define SHAPE_H

#include "../core/matrix4x4.h"
#include "../core/vector3d.h"
#include "../core/ray.h"
#include "../materials/material.h"


class Intersection;

class Shape
{
public:
    Shape() = delete;
    Shape(const Matrix4x4 &t_, Material *material_);

    // Pure virtual function makes this class Abstract class.

    // Ray/shape intersection methods
    virtual bool rayIntersect(const Ray &ray, Intersection &its) const = 0;
    virtual bool shadowRayIntersect(const Ray &ray) const = 0;

    virtual void getTextureCoordinates(const Vector3D &pointWorld, double &uOut, double &vOut) const;

	virtual void getRandomPointOnSurface(Vector3D &point_out, Vector3D &normal_out, float &proba) const = 0;

    // Return the material associated with the shape
    const Material& getMaterial() const;
	void setEmission(const Vector3D &emission_);
	bool isEmissiveShape() const;
	Vector3D getEmission() const;

protected:
    Matrix4x4 objectToWorld;
    Matrix4x4 worldToObject;
    Material *material;
	Vector3D emission;
	bool isEmissive;
};

#endif // SHAPE_H
